/* This file contains JS function for displaying editions!
    This includes the synoptic view and the textual view.
    Functions that relate to only one of them are outsourced
    to edition_surround.js and edition_synoptic.js */

/* Test browser and version because of compatibility of
-URLSearchparams
-XMLHttpRequest
*/
function doFinalBrowserTest() {
    if ($.browser.desktop) {
        if ($.browser.chrome) {
            if ($.browser.chrome.version <= 50) {errorOldVersion();}
        }
        else if ($.browser.firefox) {
            if ($.browser.firefox.version <= 45) {errorOldVersion();}
        }
		else if ($.browser.mozilla) {
            if ($.browser.mozilla.version <= 45) {errorOldVersion();}
        }
        else if ($.browser.safari) {
            if ($.browser.safari.version <= 10) {errorOldVersion();}
        }
        else {errorUnsupportedBrowser()}
    }
    else if ($.browser.mobile) {
        /* does it work for mobile?? */
        /* migrate docs here https://github.com/gabceb/jquery-browser-plugin/blob/master/dist/jquery.browser.js and here https://github.com/gabceb/jquery-browser-plugin */
    }
    else {errorUnsupportedBrowser()}
}
doFinalBrowserTest();

/* Dropdown menu: Deactivate close function when clicking on checkbox */
$("#dropdown-menu").click(function(e) {
	e.stopPropagation()
});

/* Checkbox functions for display options to trigger
visivbility of icions inside the edition text */
$(".checkbox").change(function(e) {
	e.stopPropagation();
	var id = $(this).attr("id")
	var checkValue = $(this)
		.find("input")
		.is(":checked")
	switch (id) {
		case 'placeNames':
			triggerSVGVisibility(".placeName svg");
			break;
		case 'persNames':
			triggerSVGVisibility(".persName svg");
			break;
		case 'workNames':
			triggerSVGVisibility(".ArtWork svg");
			break;
		case 'scientificComments':
			triggerSVGVisibility(".scientificComment svg");
			triggerSVGVisibility(".translationComment svg");
			triggerSVGVisibility(".editorialComment svg");
			break;
		case 'origSpelling':
			triggerSVGVisibility(".abbr-yes");
			triggerSVGVisibility(".abbr-no");
			break;
		case 'abbreviations':
			triggerSVGVisibility(".tei-add");
			break;
		case 'dates':
			triggerSVGVisibility(".tei-date svg");
			break;
		case 'showAll':
			if ($('#placeNames input').is(":checked") != checkValue) {
				$('#placeNames input').click();
			}
			if ($('#persNames input').is(":checked") != checkValue) {
				$('#persNames input').click();
			}
			if ($('#workNames input').is(":checked") != checkValue) {
				$('#workNames input').click();
			}
			if ($('#origSpelling input').is(":checked") != checkValue) {
				$('#origSpelling input').click();
			}
			if ($('#abbreviations input').is(":checked") != checkValue) {
				$('#abbreviations input').click();
			}
			if ($('#dates input').is(":checked") != checkValue) {
				$('#dates input').click();
			}
			break;
		default:
			break;
	}
});

/* General function to trigger visibility of an SVG */
function triggerSVGVisibility(classtype) {
	if (
		$(classtype).css("display") == "none"
	) {
		$(classtype).css("display", "inline")
	} else {
		$(classtype).css("display", "none")
	}
}

function triggerPageIcon(status) {

	if (status) {
		$('#pageNumberLoader').css('display', 'none');
		$('#pageNumberError').css('display', 'none');
		$('#pageNumber').css('display', 'block');
	} else {
		$('#pageNumberLoader').css('display', 'block');
		$('#pageNumberError').css('display', 'none');
		$('#pageNumber').css('display', 'none');
	}

}

function triggerPageIconError(status) {
	if (status) {
		$('#pageNumberLoader').css('display', 'none');
		$('#pageNumber').css('display', 'none');
		$('#pageNumberError').css('display', 'block');
	}
}

/* Activate tooltips (only hovering!) */
$(function() {
	$('[data-toggle="tooltip"]').tooltip({
		trigger: "hover"
	});
});

/* general function to get getParameters out of URL */
function getParameter(parameterName) {
	var result = null,
		tmp = [];
	location.search
		.substr(1)
		.split("&")
		.forEach(function(item) {
			tmp = item.split("=");
			if (tmp[0] === parameterName) result = decodeURIComponent(tmp[1]);
		});
	return result;
}

/* general function to set getParameters into URL */
function setParameter(key, value) {
	const params = new URLSearchParams(location.search);
	params.set(key, value);
	window.history.pushState({}, '', `${location.pathname}?${params}`);
}

/* The following functions relate to pagination */

function setContent(container, data) {
	$(container).html(data);
	triggerPageIcon(true);

	var number = sessionStorage.getItem('number');
	document.getElementById('pageNumber').innerHTML = number;
	sessionStorage.setItem('pbNumbers', document.getElementById('pbNumbers').innerHTML);
	sessionStorage.setItem('fwNumbers', document.getElementById('fwNumbers').innerHTML);
	checkForPossibleNeighbourPages(number);

	pastTransLoad(container,number);
}

function setEditionContent(data) {
    setContent("#edition-container",data);
}
function setTranslationContent(data) {
    setContent("#translation-container",data);
}
function setCodeContent(data) {
    $('#code-container').html(data);
	$('#code-container').parent().css("overflow-y", "scroll");
}

function fetchContent(url, functionAfterResponse) {
    fetch(url, {
        method: 'GET',
        credentials: 'include',
        mode: 'no-cors'
    })
    .then(function(response) {
        if (response.ok) {
            return response.text();
        }
        else if (response.status == 404){
            triggerPageIconError(true);
            console.log("reponse 404");
            errorFile404();
        }
        else {
            triggerPageIconError(true);
            console.log("reponse error");
            errorFileLoad(response.status);
        }
    }).then(function(string) {
        functionAfterResponse(string);
    }).catch(function(err) {
        console.log('Fetch Error', err);
    });
}

function getContent(url, functionAfterResponse, timeoutDuration) {
    /* TODO: handle timeout */

    new Promise(function(resolve, reject) {
        fetchContent(url, functionAfterResponse);
    });
}

function loadTrans(source, number, container) {
	sessionStorage.setItem('number', number);
	var url = source + "/" + source + "_p" + number + ".html";
	if (container == "#edition-container") {
	    getContent(url, setEditionContent, 5000);
	    sessionStorage.setItem('edition', source);
	}
	else if (container == "#translation-container") {
	    getContent(url, setTranslationContent, 5000);
	    sessionStorage.setItem('translation', source);
	}
	else {console.log("error: no such container")}
}

/* Check if the pagenumber coming from
    getParamter is a real number */
function checkNumber(value) {
	var er = /^-?[0-9]+$/
	return er.test(value)
}

/* Sets the new page in URL and triggers updatePage() to
    load contents if page check was OK*/
function setNewPage(newPage) {
	if (checkNumber(newPage)) {
		setParameter('page', newPage);
		triggerPageIcon(false);
		setBtnPage('forward', false);
        setBtnPage('back', false);
		updatePage();
	} else {
		console.log("Error: Page number is not a valid number");
	}
}

/* Updates page contents. Triggers loadPage() which is defined
    in edition_synoptic and edition_surround differently */
function updatePage() {
	var currentPage = getParameter('page');
	loadPage(getParameter('edition'), currentPage);
}

/* function to browse a page forward */
function pageForward() {
	var currentPage = getParameter('page');
	var temp = parseInt(currentPage) + 1;
	var newPage = temp.toString();
	setNewPage(newPage);
}

/* function to browse a page back */
function pageBack() {
	var currentPage = getParameter('page');
	var temp = parseInt(currentPage) - 1;
	var newPage = temp.toString();
	setNewPage(newPage);
}

/* Do not show buttons (Arrows) for
    next/before page if its the last or first page.
    checkForPossibleNeighbourPages triggers this
    function*/
function setBtnPage(btn, status) {
	if (btn == 'forward') {
		btn = '#btn-page-forward'
	} else if (btn == 'back') {
		btn = '#btn-page-back'
	} else {
		console.log('error: no such button')
	}

	if (status == true) {
		$(btn).removeClass("disabled");
	} else if (status == false) {
		$(btn).addClass("disabled");
	} else {
		console.log('error: no such status')
	}
}

/* checks if were on the first or last page.
    If so it should not be possible to go back
    or forward*/
function checkForPossibleNeighbourPages(number) {
    setTimeout(function() {
        try {
        	var allNumbers = sessionStorage.getItem('pbNumbers').split(',');
        	var first = allNumbers[0];
        	var last = allNumbers[allNumbers.length - 1];

        	setBtnPage('forward', true);
        	setBtnPage('back', true);

        	if (number == first) {
        		setBtnPage('back', false);
        	} else if (number == last) {
        		setBtnPage('forward', false);
        	} else {}
        }
        catch (e) {
            errorGeneral(e);
        }
    }, 200);
}

/* Adds all page numbers to the dropdownmenu */

/* Adds one page element */
function addLiElement(number, fw) {
	$('#ulPageNumbers').append('<li class="pageElement"><a href="#" onclick="setNewPage(' + number + ');this.blur();return false;"><span>' + number + ' (' + fw + ')</span></a></li>');
}

/* delets all page numbers to enable adding new ones */
function deleteAllLis() {
	var ul = document.getElementById('ulPageNumbers');
	if (ul) {
		while (ul.firstChild) {
			ul.removeChild(ul.firstChild);
		}
	}
}

/* get Items (page numbers) from sessionStorage
    ands adds for each page a element to the
    dropdown menu. */
function addAllPages() {
    /* timeout is important, because it takes
    some time because page numbers are loaded afterwards*/
	setTimeout(function() {
		deleteAllLis();
		var allNumbers;
		var allFWNumbers;
		var lang = getParameter("lang");
		var pages = "Pages";
		if (lang == "de") {pages = "Seiten"}
		else if (lang == "fr") {pages = "Pages"}
		try {
    		allNumbers = sessionStorage.getItem('pbNumbers').split(',');
    		allFWNumbers = sessionStorage.getItem('fwNumbers').split(',');
    		$('#ulPageNumbers').append('<li class="pageElement dropdown-header"><strong><span>'+pages+'</span></strong></li>');
    		for (var i = 0; i < allNumbers.length; i++) {
    			addLiElement(allNumbers[i], allFWNumbers[i])
    		}
		}
		catch (e) {
		    errorFile404();
		}
	}, 1000);
}

/* View in fullscreen */
var elem = document.documentElement;

function openFullscreen() {
	if (elem.requestFullscreen) {
		elem.requestFullscreen();
	} else if (elem.mozRequestFullScreen) {
		/* Firefox */
		elem.mozRequestFullScreen();
	} else if (elem.webkitRequestFullscreen) {
		/* Chrome, Safari and Opera */
		elem.webkitRequestFullscreen();
	} else if (elem.msRequestFullscreen) {
		/* IE/Edge */
		elem.msRequestFullscreen();
	}
}

/* Close fullscreen */
function closeFullscreen() {
	if (document.exitFullscreen) {
		document.exitFullscreen();
	} else if (document.mozCancelFullScreen) {
		/* Firefox */
		document.mozCancelFullScreen();
	} else if (document.webkitExitFullscreen) {
		/* Chrome, Safari and Opera */
		document.webkitExitFullscreen();
	} else if (document.msExitFullscreen) {
		/* IE/Edge */
		document.msExitFullscreen();
	}
}
/* Fullscreen button: toggles top navbar and footer + container-fluid */
function triggerFullscreenButton() {
	document.getElementById("icon-fullscreen").children[0].classList.toggle("fa-expand");
	document.getElementById("icon-fullscreen").children[0].classList.toggle("fa-compress");

	/* Hide/Show footer, top navbar and wrapper element */
	var footer = document.getElementById("footer");
	footer.style.display = footer.style.display === "none" ? "" : "none";
	var fullscreenWrapper = document.getElementById("fullscreenWrapper");
	fullscreenWrapper.style.display = fullscreenWrapper.style.display === "none" ? "" : "none";
	var navbar = document.getElementById("navbar-top");
	navbar.style.display = navbar.style.display === "none" ? "" : "none";
	try {
    	var placeholder = document.getElementById("navbar-placeholder");
    	placeholder.style.display = placeholder.style.display === "none" ? "" : "none";
	}
	catch (e) {console.log(e)}
}

function fullscreen() {
	if (document.getElementById("icon-fullscreen").children[0].classList.contains("fa-expand")) {
		openFullscreen();
	} else {
		closeFullscreen();
	}
	if ($("#goldenlayout").length > 0) {
    	setTimeout(function() {
            setGoldenHeight();
        }, 400);
	}
}
if (document.addEventListener) {
	document.addEventListener('webkitfullscreenchange', exitHandler, false);
	document.addEventListener('mozfullscreenchange', exitHandler, false);
	document.addEventListener('fullscreenchange', exitHandler, false);
	document.addEventListener('MSFullscreenChange', exitHandler, false);
}

function exitHandler() {
	if (document.webkitIsFullScreen === false) {
		triggerFullscreenButton();
	} else if (document.mozFullScreen === false) {
		triggerFullscreenButton();
	} else if (document.msFullscreenElement === false) {
		triggerFullscreenButton();
	} else if (document.webkitIsFullScreen === true) {
		triggerFullscreenButton();
	} else if (document.mozFullScreen === true) {
		triggerFullscreenButton();
	} else if (document.msFullscreenElement === true) {
		triggerFullscreenButton();
	}
}

function toggleLoader() {
	var yourUl = document.getElementById("loader");
	yourUl.style.display = yourUl.style.display === 'none' ? '' : 'none';
	yourUl = document.getElementById("loaderWrapper");
	yourUl.style.display = yourUl.style.display === 'none' ? '' : 'none';
}

/* Increase/Decrease Font-Size Buttons */
function getCurrentFontSize() {
    var currentFontSize;
	if ($("#goldenlayout").length == 0) {
	   currentFontSize = localStorage.getItem("fontSizeSurround");
	}
	else {
	    currentFontSize = localStorage.getItem("fontSizeSynoptic");
	}
	return currentFontSize;
}
function setCurrentFontSize(input) {
	if ($("#goldenlayout").length == 0) {
	   localStorage.setItem("fontSizeSurround",input);
	}
	else {
	    localStorage.setItem("fontSizeSynoptic",input);
	}
}
function changeFont(change) {
	var currentFontSize = getCurrentFontSize();
	if ($("#goldenlayout").length == 0) {
	   currentFontSize = localStorage.getItem("fontSizeSurround");
	}
	else {
	    currentFontSize = localStorage.getItem("fontSizeSynoptic");
	}
	var newFontSize;
	if (currentFontSize == null) {
		currentFontSize = "140"
	}

	newFontSize = (parseInt(currentFontSize) + change).toString();
	setCurrentFontSize(newFontSize);
	newFontSize = newFontSize + "%"
	if ($("#translation-container").length > 0) {
	    $("#translation-container").css("font-size", newFontSize);
	}
	if ($("#edition-container").length > 0) {
	    $("#edition-container").css("font-size", newFontSize);
	}
}
function setFontSize() {
    try {
        var fontSize = getCurrentFontSize();
        fontSize = fontSize + "%"
        if ($("#translation-container").length > 0) {
    	    $("#translation-container").css("font-size", fontSize);
	    }
	    if ($("#edition-container").length > 0) {
    	    $("#edition-container").css("font-size", fontSize);
	    }
    }
    catch (e) {console.log(e)}
}

function resetFont() {
    var defaultvalue;
    if ($("#goldenlayout").length > 0) {
        defaultvalue = 120;
    }
    else {
        defaultvalue = 140;
    }
    defaultvalue = defaultvalue + "%"
	if ($("#edition-container").length > 0) {
	    $("#edition-container").css("font-size", defaultvalue);
	}
	if ($("#translation-container").length > 0) {
	    $("#translation-container").css("font-size", defaultvalue);
    }
	setCurrentFontSize(defaultvalue);
}

function increaseFont() {
	changeFont(10);
}

function decreaseFont() {
	changeFont(-10);
}
resetFont();

function loadBrowserHistory() {
	updatePage();
}

/* A click on FA-icons and -links displays a modal */
function openRegister(textGridURI) {
	$("#myModalLabel").text(textGridURI);
	$("#myModal").modal("show");
}

function openPersName(textGridURI) {
	openRegister(textGridURI);
}

function openPlaceName(textGridURI) {
	openRegister(textGridURI);
}

function openArtWork(textGridURI) {
	openRegister(textGridURI);
}

function openScientificComment(comment) {
	openModalForComment("scientific", comment);
}

function openTranslationComment(comment) {
	openModalForComment("translation", comment);
}

function openEditorialComment(comment) {
	openModalForComment("editorial", comment);
}

function openModalForComment(type, comment) {
	$("#ModalCommentLabel").text("Note type: " + type);
	$("#ModalCommentText").text(comment);
	$("#ModalComment").modal("show");
}

/* Key events */
function checkValidPage(change) {
    change = parseInt(change);
    var availablePages = sessionStorage.getItem("pbNumbers").split(',');
    var currentPage = sessionStorage.getItem("number");
    try { currentPage = parseInt(currentPage); }
    catch (e) { console.log(e) }
    currentPage = (currentPage + change).toString();
    if (availablePages.includes(currentPage)) {
        return true;
    }
    else { return false; }
}
document.onkeydown = checkKey;
function checkKey(e) {

    e = e || window.event;

    /* For pageination */
    if (e.keyCode == '37') {
        if (checkValidPage(-1)) {
            pageBack();
        }
        else {}
    }
    else if (e.keyCode == '39') {
        if (checkValidPage(+1)) {
            pageForward();
        }
        else {}
    }
    /* else if (e.keyCode == '38') {
        console.log("up arrow");
    }
    else if (e.keyCode == '40') {
        console.log("down arrow");
    }*/

}
